/**
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */
package com.perforce.maven.scm.provider.p4.command.edit;

import java.util.List;

import org.apache.maven.scm.ScmException;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.ScmResult;
import org.apache.maven.scm.command.edit.AbstractEditCommand;
import org.apache.maven.scm.command.edit.EditScmResult;
import org.apache.maven.scm.provider.ScmProviderRepository;

import com.perforce.maven.scm.provider.p4.command.P4Command;
import com.perforce.maven.scm.provider.p4.util.P4Utils;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.option.client.EditFilesOptions;

/**
 * The class uses the Perforce edit command to open existing files for edit.
 */
public class P4EditCommand
    extends AbstractEditCommand
    implements P4Command
{

    /** The Constant command. */
    public static final String command = "Perforce edit command";

    private IClient client;

    public void setClient( IClient client )
    {
        this.client = client;
    }

    /**
     * @see org.apache.maven.scm.command.edit.AbstractEditCommand#executeEditCommand
     *      (org.apache.maven.scm.provider.ScmProviderRepository, org.apache.maven.scm.ScmFileSet)
     */
    protected ScmResult executeEditCommand( ScmProviderRepository repo, ScmFileSet files )
        throws ScmException
    {

        P4EditResult p4Result = new P4EditResult();

        List<IFileSpec> fileSpecs = P4Utils.scmFileSet2P4FileSpecs( files );

        if ( !fileSpecs.isEmpty() )
        {
            try
            {
                EditFilesOptions editFilesOptions = new EditFilesOptions();
                List<IFileSpec> retFileSpecs = client.editFiles( fileSpecs, editFilesOptions );

                p4Result.process( retFileSpecs );
            }
            catch ( P4JavaException e )
            {
                throw new ScmException( e.getLocalizedMessage(), e );
            }
        }

        return new EditScmResult( command, p4Result.isSuccess() ? "Edit successful" : "Unable to edit",
                                  p4Result.getCommandOutput(), p4Result.isSuccess() );
    }

}
